@@ -3,7 +3,7 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Control-change based RtController filters
 
-our $VERSION = '0.0300';
+our $VERSION = '0.0301';
 
 use v5.36;
 
@@ -151,44 +151,38 @@ sub stair_step ($self, $device, $dt, $event) {
 
     my ($ev, $chan, $ctl, $val) = $event->@*;
 
+    my $it = Iterator::Breathe->new(
+        bottom => $self->range_bottom,
+        top    => $self->range_top,
+    );
+
     $self->running(1);
 
-    my $value  = $self->initial_point;
-    my @values = ($self->range_bottom .. $self->range_top);
-
+    my $value     = $self->initial_point;
     my $direction = 1; # up
 
-    my $up   = $self->step_up;
-    my $down = $self->step_down;
-
     while (!$self->stop) {
         my $cc = [ 'control_change', $self->channel, $self->control, $value ];
         $self->rtc->send_it($cc);
 
+        # compute the stair-stepping
         if ($direction) {
-            $value = ($value + $self->step_up) < $self->range_top ? $value + $self->step_up
-                                                                  : $self->range_top;
+            $it->step($self->step_up);
         }
         else {
-            $value = ($value - $self->step_down) > $self->range_bottom ? $value - $self->step_down
-                                                                       : $self->range_bottom;
+            $it->step(- $self->step_down);
         }
 
         # toggle the stair-step direction
         $direction = !$direction;
 
-        # flip up to down and vice-versa
-        if ($value >= $self->range_top) {
-            $value = $self->range_top;
-            $self->step_up(-$up);
-            $self->step_down(-$down);
-        }
-        if ($value <= $self->range_bottom) {
-            $value = $self->range_bottom;
-            $self->step_up($up);
-            $self->step_down($down);
-        }
+        # iterate the breathing
+        $it->iterate;
+        $value = $it->i;
+        $value = $self->range_top    if $value >= $self->range_top;
+        $value = $self->range_bottom if $value <= $self->range_bottom;
 
+        # pause until the next iteration
         usleep $self->time_step;
     }
 
@@ -209,7 +203,7 @@ MIDI::RtController::Filter::CC - Control-change based RtController filters
 
 =head1 VERSION
 
-version 0.0300
+version 0.0301
 
 =head1 SYNOPSIS
 
@@ -396,6 +390,9 @@ the B<range_bottom> and B<range_top>, and sends that as the value of a
 B<control> change message, over the MIDI B<channel>, every iteration,
 until B<stop> is seen.
 
+The B<initial_point> is used as the first CC# message, then the
+randomization takes over.
+
 Passing C<all> means that any MIDI event will cause this filter to be
 triggered.
 
@@ -403,10 +400,11 @@ triggered.
 
   $control->add_filter('stair_step', all => $filter->curry::stair_step);
 
-This filter sets the B<running> flag, chooses a number starting at the
-B<initial_point>, adds B<step_up> or subtracts B<step_down>
-successively, sending that value as a B<control> change message, over
-the MIDI B<channel>, every iteration, until B<stop> is seen.
+This filter sets the B<running> flag, uses the B<initial_point> for
+the fist CC# message, then adds B<step_up> or subtracts B<step_down>
+from that number successively, sending the value as a B<control>
+change message, over the MIDI B<channel>, every iteration, until
+B<stop> is seen.
 
 Passing C<all> means that any MIDI event will cause this filter to be
 triggered.
